pro DAVE1Setup

; Define useful DAVE1 global variables (defined as system variables)
davesysvars


; No matter what, !dave_defaults is defined in the next few lines of code.
if (file_test(!defaults_file)) then begin
    restore,!defaults_file ; restore pdd from file
    if (strcmp(vartype((*pdd).classic),'BYTE')) then begin
      ; if classic field is defined as a byte then we need to redefine
      ; the restored structure because of a bug introduced in IDL 8 
      ; which causes sav files containing a byte variable to be 
      ; incompatible and cannot be read by earlier versions of IDL.
      ; Ensure classic and show_choice fields are defined as integers and not byte
      dave_defs = {classic:1,show_choice:1}
      void = dave_defs_checkout(dave_defs)   ; define remaining fields
      
      ntags = n_tags(*pdd)
      tnames = tag_names(*pdd)
      for i=0,ntags-1 do begin
         ; copy the restored values over to the new structure
         ; except for show_choice which had previously been initialized to 255B
         ; and therefore not a meaningful value for an integer expected to be 0|1
         index = where(tag_names(dave_defs) eq tnames[i], present)
         allowed = ~strcmp(tnames[i],'SHOW_CHOICE')
         if (present && allowed) then dave_defs.(index) = (*pdd).(i)
      endfor
      
    endif else begin
      dave_defs=*pdd              
      void = dave_defs_checkout(dave_defs)
    endelse
    
    ptr_free, pdd
endif else begin
    dave_defs = {datDir:!home_dir,workDir:!home_dir,PDFReader:'', $ ; RTA -change thisDir->!home_dir as default workDir
                 web_browser:'',classic:1,show_choice:1,tree_menu_pointer:ptr_new(/allocate_heap)}
    void = dave_defs_checkout(dave_defs)
endelse

if (strlen(strtrim(dave_defs.web_browser)) eq 0) then $
  dave_defs.web_browser = locate_web_browser()

pdd = ptr_new(dave_defs)   ; Memory leak!!! Need to figure out where to cleanup!
defsysv,'!dave_defaults', pdd 

; store !dave_defaults on disk in file !defaults_file
save,filename=!defaults_file,pdd


end


;===============================================================================
; fdaveSELpdfreader
;
; PURPOSE:
;   Event handler. Let user specify a PDF Reader app. The specified
;   value is stored in a global variable and also saved for future DAVE
;   sessions.
;
; PARAMETERS:
;   event [in] - event structure.
;
pro fdaveSELpdfreader,event

dave_defs=*!dave_defaults
PDFReader=dave_defs.PDFReader
;print,"Current pdfreader is ",PDFReader

; check and then exit if reader's selected reader is okay
readerNotOkay = 'Yes'
if (strtrim(PDFReader,2) ne '') then begin
    message_text = ['Your PDF reader is currently set to:',PDFReader, $
                    'Are you sure you want to change this setting?']
    readerNotOkay = dialog_message(message_text,/question,/default_no, $
                                   title='PDF Reader Setting')
    readerBaseDir = strmid(PDFReader,0,strpos(PDFReader,!delimiter,/reverse_search))
endif
if (readerNotOkay eq 'No') then return ; nothing to do

; let user select select a reader
daveDeSensitizeButtons,event
if n_elements(event.top) gt 0 then begin
    acroread = dialog_pickfile(dialog_parent = event.top,/read, $
                               title="Please select your PDF reader", $
                               path=readerBaseDir, $
                               file = PDFReader)
endif else begin
    acroread = dialog_pickfile(/read, title="Please select your PDF reader", $
                               path=readerBaseDir, $
                               file = PDFReader)
endelse

if (acroread eq '' or acroread eq ' ') then begin
    daveSensitizeButtons,event
    return
endif
dave_defs.PDFReader=acroread
pdd=!dave_defaults
ptr_free,pdd
pdd=ptr_new(dave_defs)



    save,filename=!defaults_file,pdd


defsysv,'!dave_defaults',pdd
print,"New pdfreader is ",(*!dave_defaults).PDFReader
daveSensitizeButtons,event
return
end
;-------------------------------------------------------------------------------



;===============================================================================
; fdaveSELbrowser
;
; PURPOSE:
;   Event handler. Let user specify a web browser. The specified
;   value is stored in a global variable and also saved for future DAVE
;   sessions.
;
; PARAMETERS:
;   event [in] - event structure.
;
pro fdaveSELbrowser,event
; Added this routine: RMD, February 7, 2004
dave_defs=*!dave_defaults
web_browser = dave_defs.web_browser

; check and then exit if reader's selected browser is okay
readerNotOkay = 'Yes'
if (strtrim(web_browser,2) ne '') then begin
    message_text = ['Your web_browser reader is currently set to:',web_browser, $
                    'Are you sure you want to change this setting?']
    readerNotOkay = dialog_message(message_text,/question,/default_no, $
                                   title='Web browser Setting')
    readerBaseDir = strmid(web_browser,0,strpos(web_browser,!delimiter,/reverse_search))
endif
if (readerNotOkay eq 'No') then return ; nothing to do

; let user select select a reader
daveDeSensitizeButtons,event
if n_elements(event.top) gt 0 then begin
    browser = dialog_pickfile(dialog_parent = event.top,/read, $
                              title="Please select your web browser", $
                              path=readerBaseDir, $
                              file = web_browser)
endif else begin
    browser = dialog_pickfile(/read, title="Please select your web_browser", $
                              path=readerBaseDir, $
                              file = web_browser)
endelse

if ((browser eq '') or (browser eq ' ')) then begin
    daveSensitizeButtons,event
    return
endif
dave_defs.web_browser = browser
pdd=!dave_defaults
ptr_free,pdd
pdd=ptr_new(dave_defs)



    save,filename=!defaults_file,pdd


defsysv,'!dave_defaults',pdd
print,"New web browser is ",(*!dave_defaults).web_browser
daveSensitizeButtons,event
return
end
;-------------------------------------------------------------------------------


;===============================================================================
; dave_defs_checkout
;
; PURPOSE:
;   Certain user settings are saved between DAVE sessions. This
;   routine ensures that the saved structure has all the proper
;   fields.
;
; PARAMETERS:
;   dave_defs [in] - An IDL structure conatining the session variables
;                    fields to be saved.
;
; RETURN VALUE:
;   status flag. Insignificant.
;
function dave_defs_checkout, dave_defs
; RTA - completely revamp code to insert the necessary tags in the
;       dave_defs structure instead of simply checking.
;
; "WEB_BROWSER" tag added 2-4-04 (RMD)
cd,current=cwd
tags_present = strupcase(tag_names(dave_defs))
; insert 'datDir' field if necessary
void = where(tags_present eq 'DATDIR', count)
if (count le 0) then dave_defs = create_struct(dave_defs, 'datDir', cwd)
; insert 'workDir' field if necessary
void = where(tags_present eq 'WORKDIR', count)
if (count le 0) then dave_defs = create_struct(dave_defs, 'workDir', cwd)
; insert PDF Reader field if necessary
void = where(tags_present eq 'PDFREADER', count)
if (count le 0) then dave_defs = create_struct(dave_defs, 'PDFReader', '')
; insert Web Browser field if necessary
void = where(tags_present eq 'WEB_BROWSER', count)
if (count le 0) then dave_defs = create_struct(dave_defs, 'web_browser', '')
; insert classic field if necessary
void = where(tags_present eq 'CLASSIC', count)
if (count le 0) then dave_defs = create_struct(dave_defs, 'classic', 1)
; insert show_choice field if necessary
void = where(tags_present eq 'SHOW_CHOICE', count)
if (count le 0) then dave_defs = create_struct(dave_defs, 'show_choice', 1)
; insert tree menu pointer field if necessary
void = where(tags_present eq 'TREE_MENU_POINTER', count)
if (count le 0) then dave_defs = create_struct(dave_defs, 'tree_menu_pointer', ptr_new(/allocate_heap))

; insert 'psd_a4_spacing' field if necessary
void = where(strlowcase(tags_present) eq 'psd_a4_spacing', count)
if (count le 0) then dave_defs = create_struct(dave_defs, 'psd_a4_spacing','')


; insert 'psd_channel_Eff' field if necessary
void = where(strlowcase(tags_present) eq 'psd_channel_eff', count)
if (count le 0) then dave_defs = create_struct(dave_defs, 'psd_channel_eff','')

; insert 'psd_Ef_spacing' field if necessary
void = where(strlowcase(tags_present) eq 'psd_ef_spacing', count)
if (count le 0) then dave_defs = create_struct(dave_defs, 'psd_ef_spacing', '')


return, 0

end
;-------------------------------------------------------------------------------
