; $Id: dm_calc_deteff.pro,v 1.7 2009/12/02 16:37:15 ymqiu Exp $
;#######################################################################
;
; NAME:
;  dm_calc_deteff
;
; PURPOSE:
;   calculate detector efficiency
;
; CATEGORY:
;  dcs_mslice
;
; AUTHOR:
;  Yiming Qiu
;  NIST Center for Neutron Research
;  100 Bureau Drive, Gaithersburg, MD 20899-6102
;  United States
;  yiming.qiu@nist.gov
;  December, 2009
;
; LICENSE:
;  The software in this file is written by an employee of
;  National Institute of Standards and Technology
;  as part of the DAVE software project.
;
;  The DAVE software package is not subject to copyright protection
;  and is in the public domain. It should be considered as an
;  experimental neutron scattering data reduction, visualization, and
;  analysis system. As such, the authors assume no responsibility
;  whatsoever for its use, and make no guarantees, expressed or
;  implied, about its quality, reliability, or any other
;  characteristic. The use of certain trade names or commercial
;  products does not imply any endorsement of a particular product,
;  nor does it imply that the named product is necessarily the best
;  product for the stated purpose. We would appreciate acknowledgment
;  if the DAVE software is used or if the code in this file is
;  included in another product.
;
;#######################################################################

; parameter:
;   qty:  [ne,ndet], value will be changed if maskdet exists
; keyword:
;   fit:  if set, fit by an gaussian and use the data in the gaussian fucntion range
;   maskdet: detector indices to be masked, those detectors will be given a value of 1.0
function dm_calc_deteff,qty,fit=fit,maskdet=maskdet,debug=debug
    if n_elements(maskdet) ne 0 then begin
       qty[*,maskdet] = 0.0
    endif
    if keyword_set(fit) then begin
       ;first fit the det-sum and find out the elastic range
       y = total(qty,2,/double)
       dm_gaussfit,y,ny=ny,params=params,fitnotgood=fitnotgood,debug=debug
       if fitnotgood then begin
          eff  = total(qty,1,/double)
       endif else begin
          tmp  = shift(qty,ny/2-params[1],0)
          istart = (0>(ny/2-fix(2.5*params[2])))
          iend = ((ny-1)<(ny/2+fix(2.5*params[2])))
          tmp  = tmp[istart:iend,*] 
          eff  = total(tmp,1,/double)
       endelse
    endif else begin
       eff = total(qty,1,/double)
    endelse
    index0 = where(eff eq 0,count,complement=index1)
    if count ne 0 then begin
       minv = min(eff[index1])
       eff[index0] = minv
    endif
    eff = float(eff/mean(eff,/double))
    if n_elements(maskdet) ne 0 then eff[maskdet] = 1.0
    return, eff
end