; $Id: dm_choose_slicedata.pro,v 1.18 2010/07/22 14:38:02 ymqiu Exp $
;#######################################################################
;
; NAME:
;  dm_choose_slicedata
;
; PURPOSE:
;  choose slice data
;
; CATEGORY:
;  dcs_mslice
;
; AUTHOR:
;  Yiming Qiu
;  NIST Center for Neutron Research
;  100 Bureau Drive, Gaithersburg, MD 20899-6102
;  United States
;  yiming.qiu@nist.gov
;  July, 2010
;
; LICENSE:
;  The software in this file is written by an employee of
;  National Institute of Standards and Technology
;  as part of the DAVE software project.
;
;  The DAVE software package is not subject to copyright protection
;  and is in the public domain. It should be considered as an
;  experimental neutron scattering data reduction, visualization, and
;  analysis system. As such, the authors assume no responsibility
;  whatsoever for its use, and make no guarantees, expressed or
;  implied, about its quality, reliability, or any other
;  characteristic. The use of certain trade names or commercial
;  products does not imply any endorsement of a particular product,
;  nor does it imply that the named product is necessarily the best
;  product for the stated purpose. We would appreciate acknowledgment
;  if the DAVE software is used or if the code in this file is
;  included in another product.
;
;#######################################################################

; parameters:
;   None
; keywords:
;   (xyz)dat: data to be returned
;   zerr:     error bar of zdat
;   (xy)tit:  label of the data
;   zran:     for z data only, contour plot color bar range
;   zmin:     for z data only
pro dcs_mslice::dm_choose_slicedata,xdat=xdat,ydat=ydat,zdat=zdat,zerr=zerr,zran=zran,xtit=xtit,ytit=ytit,ztit=ztit,zmin=zmin,zmax=zmax,$
    pran_invalid=pran_invalid,tran_invalid=tran_invalid,debug=debug
    if keyword_set(debug) then begin
       print,'&&choose slice data...'
       current=systime(/sec)
    endif
    
    nogrid = [0,0]
    if self.samp_typ eq -1 then begin ;general multiple-d data
       zdat = reform((*self.dataPtr).data[*,0])
       ztit = (*self.dataPtr).label[0]+((strlen((*self.dataPtr).unit[0]) ne 0)?' ('+(*self.dataPtr).unit[0]+')':'') 
       ndat = n_elements(zdat)
       if arg_present(zerr) then begin
          zerr    = reform((*self.dataPtr).data[*,1])
          ok_zerr = 1b
       endif else ok_zerr = 0b
       ok_ewid = 0b
       
       ;choose x and y data
       self->dm_choose_trajdata,self.dispsel_h+2,data=xdat,label=xtit,is_E=is_E_x,debug=debug
       self->dm_choose_trajdata,self.dispsel_v+2,data=ydat,label=ytit,is_E=is_E_y,debug=debug
       
       ;thickness  
       ndat     = n_elements((*self.dataPtr).data[*,0])
       nitem    = n_elements((*self.dataPtr).label)
       sel_list = (*self.dataPtr).label[2:nitem-1]
       if nitem gt 4 then tran_invalid = bytarr(nitem-4)
       for i=0,nitem-5 do begin
           if (total(finite((*self.dispsel_tft)[i,*])) ne 0) then begin
              self->dm_choose_trajdata,(*self.dispsel_t)[i]+2,index=index1,range=reform((*self.dispsel_tft)[i,*]),$
                    empty=empty,debug=debug
              if empty then begin
                 emptyitem = sel_list[(*self.dispsel_t)[i]]
                 index1 = -1 & tmp = temporary(index1)
                 tran_invalid[i] = 1b
              endif else if n_elements(index1) eq ndat then begin
                 index1 = -1 & tmp = temporary(index1)
              endif else begin
                 if n_elements(index) eq 0 then index = index1 $
                 else begin
                    index = dm_common(index,index1,/no_copy)
                 endelse
              endelse
           endif
       endfor
       if n_elements(index) ne 0 then begin
          if index[0] eq -1 then begin
             emptyitem = 'combined thickness'
             if n_elements(tran_invalid) ne 0 then tran_invalid[*] = 1b
          endif else begin
             xdat = xdat[index]
             ydat = ydat[index]
             zdat = zdat[index]
             if ok_zerr then zerr = zerr[index]
             index = -1
          endelse
       endif
       if is_E_x then begin
          if finite(self.disp_hstep) eq 0 then begin
             nogrid[0] = 1 
          endif else begin
             if self.disp_hstep eq 0 then nogrid[0] = 1 
          endelse
          if nogrid[0] then uniq_E_x = xdat[uniq(xdat,sort(xdat))] 
       endif
       if is_E_y then begin
          if finite(self.disp_vstep) eq 0 then begin
             nogrid[1] = 1  
          endif else begin
             if self.disp_vstep eq 0 then nogrid[1] = 1 
          endelse
          if nogrid[1] then uniq_E_y = ydat[uniq(ydat,sort(ydat))] 
       endif
    endif else begin     
       zdat = (*self.projStrPtr).qty
       ztit = self.intn_lab
       ndat = n_elements(zdat)
       if arg_present(zerr) then begin
          zerr    = (*self.projStrPtr).err
          ok_zerr = 1b
       endif else ok_zerr = 0b

       ;ewid
       if self.eint_yn and (self.samp_typ ne 2) then $
          self->dm_choose_trajdata,-2,data=ewid
       ok_ewid = (n_elements(ewid) eq ndat)

       viewlbs = [self.view_u1lab[(self.samp_typ ge 1)],self.view_u2lab[(self.samp_typ ge 1)],'E']
    
       if self.samp_typ eq 2 then ids=[0,1,2] else ids  = [0,1,3]
       x_id = [self.view_u1typ,ids[(self.disp_perp+1) mod 3],0,ids[(self.disp_perp+1) mod 3]]
       y_id = [self.view_u2typ,ids[(self.disp_perp+2) mod 3],1,ids[(self.disp_perp+2) mod 3]]
       if self.extravaxis_yn then begin
          x_id[2] = ids[(self.disp_perp+1) mod 3]
          y_id[2] = ids[(self.disp_perp+2) mod 3]
          if self.samp_typ eq 2 then viewlbs[2] = self.view_u3lab
       endif
    
       ;choose x and y data
       self->dm_choose_trajdata,x_id[self.samp_typ],data=xdat,label=xtit,is_E=is_E_x,debug=debug
       self->dm_choose_trajdata,y_id[self.samp_typ],data=ydat,label=ytit,is_E=is_E_y,debug=debug
       if is_E_x then begin
          if finite(self.disp_hstep) eq 0 then begin
             nogrid[0] = 1 
          endif else begin
             if self.disp_hstep eq 0 then nogrid[0] = 1 
          endelse
          if nogrid[0] then uniq_E_x = xdat[*,0,0]
       endif
       if is_E_y then begin
          if finite(self.disp_vstep) eq 0 then begin
             nogrid[1] = 1  
          endif else begin
             if self.disp_vstep eq 0 then nogrid[1] = 1 
          endelse
          if nogrid[1] then uniq_E_y = ydat[*,0,0]
       endif
    
       ;perp thickness
       pthick = ((self.samp_typ eq 1) or (self.samp_typ eq 3) or ((self.samp_typ eq 2) and self.extravaxis_yn))
       if (total(finite(self.disp_pran)) ne 0) and pthick then begin
          self->dm_choose_trajdata,ids[self.disp_perp],index=index1,range=self.disp_pran,empty=empty,debug=debug
          if empty then begin
             emptyitem = viewlbs[self.disp_perp]
             index1 = -1 & tmp = temporary(index1)
             pran_invalid = 1b
          endif else if n_elements(index1) eq ndat then begin
             index1 = -1 & tmp = temporary(index1)
          endif
       endif
  
       ;extra viewing axis thickness
       ethick = (((self.samp_typ eq 1) or (self.samp_typ eq 3)) and self.extravaxis_yn)
       if (total(finite(self.disp_tran)) ne 0) and ethick then begin
          self->dm_choose_trajdata,2,index=index2,range=self.disp_tran,empty=empty,debug=debug
          if empty then begin
             if n_elements(emptyitem) eq 0 then emptyitem = self.view_u3lab $
             else emptyitem = emptyitem+' and '+self.view_u3lab
             index2 = -1 & tmp = temporary(index2)
             tran_invalid = 1b
          endif else if n_elements(index2) eq ndat then begin
             index2 = -1 & tmp = temporary(index2)
          endif 
       endif

       n_index1 = n_elements(index1)
       n_index2 = n_elements(index2)
       if (n_index1 ne 0) or (n_index2 ne 0) then begin
          if n_index1 eq 0 then index = temporary(index2) $
          else if n_index2 eq 0 then index=temporary(index1) $
          else index = dm_common(index1,index2,/no_copy)
          if index[0] eq -1 then begin
             emptyitem = viewlbs[self.disp_perp]+' and '+self.view_u3lab
             tran_invalid = 1b
             pran_invalid = 1b
          endif else if n_elements(zdat) ne n_elements(index) then begin
             xdat = xdat[index]
             ydat = ydat[index]
             zdat = zdat[index]
             if ok_zerr then zerr = zerr[index]
             if ok_ewid then ewid = ewid[index]
          endif
          index = -1
       endif
    endelse

    if n_elements(emptyitem) ne 0 then $
       ok = dialog_message(['The specified '+emptyitem+' range covers no data.', $
               'The entire possible range is used instead.'],dialog_parent=self.tlb)

    ;horizontal range and vertical range
    n_xran = total(finite(self.disp_hran))
    n_yran = total(finite(self.disp_vran))
    if n_xran eq 2 then begin
       if self.disp_hran[1] gt self.disp_hran[0] then begin
          index = where((xdat le self.disp_hran[1]) and (xdat ge self.disp_hran[0]),count)
          if count gt 0 then begin
             xdat = xdat[index]
             ydat = ydat[index]
             zdat = zdat[index]
             if ok_ewid then ewid = ewid[index]
             if ok_zerr then zerr = zerr[index]
             index = -1
          endif
       endif
    endif
    if n_yran eq 2 then begin
       if self.disp_vran[1] gt self.disp_vran[0] then begin
          index  = where((ydat le self.disp_vran[1]) and (ydat ge self.disp_vran[0]),count)
          if count gt 0 then begin
             xdat = xdat[index]
             ydat = ydat[index]
             zdat = zdat[index]
             if ok_ewid then ewid = ewid[index]
             if ok_zerr then zerr = zerr[index]
             index = -1
          endif
       endif
    endif
    if n_xran lt 2 then begin
       if finite(self.disp_hran[0]) then begin
          index = where(xdat ge self.disp_hran[0],count)
          if count gt 0 then begin
             xdat = xdat[index]
             ydat = ydat[index]
             zdat = zdat[index]
             if ok_ewid then ewid = ewid[index]
             if ok_zerr then zerr = zerr[index]
             index = -1
          endif
       endif
       if finite(self.disp_hran[1]) then begin
          index = where(xdat le self.disp_hran[1],count)
          if count gt 0 then begin
             xdat = xdat[index]
             ydat = ydat[index]
             zdat = zdat[index]
             if ok_ewid then ewid = ewid[index]
             if ok_zerr then zerr = zerr[index]
             index = -1
          endif
       endif
    endif
    if n_yran lt 2 then begin
       if finite(self.disp_vran[0]) then begin
          index = where(ydat ge self.disp_vran[0],count)
          if count gt 0 then begin
             xdat = xdat[index]
             ydat = ydat[index]
             zdat = zdat[index]
             if ok_ewid then ewid = ewid[index]
             if ok_zerr then zerr = zerr[index]
             index = -1
          endif
       endif
       if finite(self.disp_vran[1]) then begin
          index = where(ydat le self.disp_vran[1],count)
          if count gt 0 then begin
             xdat = xdat[index]
             ydat = ydat[index]
             zdat = zdat[index]
             if ok_ewid then ewid = ewid[index]
             if ok_zerr then zerr = zerr[index]
             index = -1
          endif
       endif
    endif

    ;rebinning according to the steps
    ;rebin even if hstep and vstep are not specified for faster contour plotting
    xmax   = max(xdat, min=xmin)
    ymax   = max(ydat, min=ymin)
    hstep  = (xmax-xmin)/400.
    vstep  = (ymax-ymin)/400.
    if nogrid[0] then begin
       index = where((uniq_E_x ge xmin) and (uniq_E_x le xmax),count)
       if count ne 0 then uniq_E_x = uniq_E_x[index]
       extra = {uniq_xval:uniq_E_x} 
    endif
    if nogrid[1] then begin
       index = where((uniq_E_y ge ymin) and (uniq_E_y le ymax),count)
       if count ne 0 then uniq_E_y = uniq_E_y[index]
       if n_elements(extra) eq 0 then $
          extra = {uniq_yval:uniq_E_y} $
       else $
          extra = create_struct('uniq_yval',uniq_E_y,extra)
    endif
    index = -1
    if finite(self.disp_hstep) then begin
       if (self.disp_hstep ne 0) then begin
          hstep = abs(self.disp_hstep)
       endif
    endif
    if finite(self.disp_vstep) then begin
       if (self.disp_vstep ne 0) then begin
          vstep = abs(self.disp_vstep)
       endif
    endif
    if ok_ewid then begin
       dE = 0
       if is_E_x then dE = hstep
       if is_E_y then dE = vstep
       if ((self.samp_typ eq 1) or (self.samp_typ eq 3)) and (self.disp_perp eq 2) then begin
             tmp_ran = self.disp_pran
             emax = max((*self.projStrPtr).en,min=emin)
             if finite(tmp_ran[0]) eq 0 then tmp_ran[0]=-1e30
             if finite(tmp_ran[1]) eq 0 then tmp_ran[1]=1e30
             if tmp_ran[0] lt emin then tmp_ran[0]=emin
             if tmp_ran[1] gt emax then tmp_ran[1]=emax
             if tmp_ran[1] gt tmp_ran[0] then dE=tmp_ran[1]-tmp_ran[0] $
             else dE=emax-emin
       endif
       if dE le 0 then dE=1.0
    endif

    checkfinite = ((self.ftype eq 1) or (self.ftype eq 2) or (strlen(self.userf_str) ne 0))
    dm_stepgrid_bin,hstep,vstep,xdat,ydat,zdat,zerr=zerr,avgsum=self.bin_avgsum,bintr=self.binthreshold,$
        conststep=self.bin_conststep,ewid=ewid,debug=debug,checkfinite=checkfinite,extlib=self.bin_extlib,_extra=extra

    if ok_ewid then zdat=zdat*dE

    ;smooth regular grid data
    index  = where(~finite(zdat),count)     ;bins with no data
    refill = (count ne 0)                   ;flag for refilling
    if self.disp_smooth gt 0 then begin     ;smooth data 
       tmp = size(zdat,/dimension)
       if (n_elements(tmp) gt 1) and (min(tmp) gt 3) then begin
          if ok_zerr then zerr = zerr^2
          if float(!version.release) ge 6.2 then begin
             for i=0, self.disp_smooth-1 do begin
                 zdat = convol(zdat,self.disp_sm_kernel,/edge_truncate,/nan,/normalize)
                 if ok_zerr then zerr = convol(zerr,self.disp_sm_kernel,/edge_truncate,/nan,/normalize)
                 if refill then begin
                    zdat[index] = !values.f_nan
                    if ok_zerr then zerr[index] = !values.f_nan
                 endif
             endfor
          endif else begin
             for i=0, self.disp_smooth-1 do begin
                 ;put 0 in bins with no data
                 if refill then begin
                    zdat[index] = 0.0
                    if ok_zerr then zerr[index] = 0.0
                 endif
                 zdat = convol(zdat,self.disp_sm_kernel,total(self.disp_sm_kernel),/edge_truncate)
                 if ok_zerr  then zerr = convol(zerr,self.disp_sm_kernel,total(self.disp_sm_kernel),/edge_truncate)
             endfor
             if refill then begin
                zdat[index] = !values.f_nan
                if ok_zerr then zerr[index] = !values.f_nan
             endif
          endelse
          if ok_zerr then zerr = sqrt(zerr)
       endif
    endif
    index = -1
    ;intensity range
    oldzmax  = max(zdat,min=oldzmin,/nan)
    zmax = oldzmax & zmin = oldzmin
    if finite(self.disp_iran[0]) then $
       if self.disp_iran[0] lt oldzmax then zmin=self.disp_iran[0]
    if finite(self.disp_iran[1]) then $
       if self.disp_iran[1] gt oldzmin then zmax=self.disp_iran[1]
    zran = [zmin,zmax]
    if keyword_set(debug) then begin
       help,xdat,nogrid,uniq_E_x,ydat,uniq_E_y
       print,'&&choose slice data finished in ',systime(/sec)-current,' sec.'
    endif
end