; $Id: dm_choose_volumedata.pro,v 1.11 2010/07/22 14:38:02 ymqiu Exp $
;#######################################################################
;
; NAME:
;  dm_choose_volumedata
;
; PURPOSE:
;  choose volume data
;
; CATEGORY:
;  dcs_mslice
;
; AUTHOR:
;  Yiming Qiu
;  NIST Center for Neutron Research
;  100 Bureau Drive, Gaithersburg, MD 20899-6102
;  United States
;  yiming.qiu@nist.gov
;  July, 2010
;
; LICENSE:
;  The software in this file is written by an employee of
;  National Institute of Standards and Technology
;  as part of the DAVE software project.
;
;  The DAVE software package is not subject to copyright protection
;  and is in the public domain. It should be considered as an
;  experimental neutron scattering data reduction, visualization, and
;  analysis system. As such, the authors assume no responsibility
;  whatsoever for its use, and make no guarantees, expressed or
;  implied, about its quality, reliability, or any other
;  characteristic. The use of certain trade names or commercial
;  products does not imply any endorsement of a particular product,
;  nor does it imply that the named product is necessarily the best
;  product for the stated purpose. We would appreciate acknowledgment
;  if the DAVE software is used or if the code in this file is
;  included in another product.
;
;#######################################################################

; parameters:
;   None
; keywords:
;   (xyzi)dat: data to be returned
;   ierr:      error bar of idat
;   (xyzi)tit: label of the data
;   iran:      for i data only, volume plot color bar range
pro dcs_mslice::dm_choose_volumedata,xdat=xdat,ydat=ydat,zdat=zdat,idat=idat,ierr=ierr,iran=iran,xtit=xtit,ytit=ytit,ztit=ztit,itit=itit,$
    tran_invalid=tran_invalid,debug=debug
    if keyword_set(debug) then begin
       print,'&&choose volume data...'
       current=systime(/sec)
    endif
    nogrid = [0,0,0]
    if self.samp_typ eq -1 then begin
       idat = reform((*self.dataPtr).data[*,0])
       itit = (*self.dataPtr).label[0]+((strlen((*self.dataPtr).unit[0]) ne 0)?' ('+(*self.dataPtr).unit[0]+')':'') 
       ndat = n_elements(idat)
       if arg_present(ierr) then begin
          ierr    = reform((*self.dataPtr).data[*,1])
          ok_ierr = 1b
       endif else ok_ierr = 0b
       ok_ewid = 0b
       
       ;choose x, y, and z data
       self->dm_choose_trajdata,self.volsel_x+2,data=xdat,label=xtit,debug=debug,is_E=is_E_x
       self->dm_choose_trajdata,self.volsel_y+2,data=ydat,label=ytit,debug=debug,is_E=is_E_y
       self->dm_choose_trajdata,self.volsel_z+2,data=zdat,label=ztit,debug=debug,is_E=is_E_z

       ;thickness 
       nitem    = n_elements((*self.dataPtr).label)
       sel_list = (*self.dataPtr).label[2:nitem-1]
       if nitem gt 5 then tran_invalid = bytarr(nitem-5)
       for i=0,nitem-6 do begin
           if (total(finite((*self.volsel_tft)[i,*])) ne 0) then begin
              self->dm_choose_trajdata,(*self.volsel_t)[i]+2,index=index1,range=reform((*self.volsel_tft)[i,*]),$
                    empty=empty,debug=debug
              if empty then begin
                 if n_elements(emptyitem) eq 0 then emptyitem = sel_list[(*self.volsel_t)[i]] $
                 else emptyitem = emptyitem+' and '+sel_list[(*self.volsel_t)[i]]
                 index1 = -1 & tmp = temporary(index1)
                 tran_invalid[i] = 1b
              endif else if n_elements(index1) eq ndat then begin
                 index1 = -1 & tmp = temporary(index1)
              endif else begin
                 if n_elements(index) eq 0 then index = index1 $
                 else begin
                    index = dm_common(index,index1,/no_copy)
                 endelse
              endelse
           endif
       endfor
       if n_elements(index) ne 0 then begin
          if index[0] eq -1 then begin
             emptyitem = 'combined thickness'
             if n_elements(tran_invalid) ne 0 then tran_invalid[*] = 1b
          endif else begin
             xdat = xdat[index]
             ydat = ydat[index]
             zdat = zdat[index]
             idat = idat[index]
             if ok_ierr then ierr = ierr[index]
             index = -1
          endelse
       endif
       if is_E_x then begin
          if finite(self.vol_xstep) eq 0 then begin
             nogrid[0] = 1 
          endif else begin
             if self.vol_xstep eq 0 then nogrid[0] = 1 
          endelse
          if nogrid[0] then uniq_E_x = xdat[uniq(xdat,sort(xdat))] 
       endif
       if is_E_y then begin
          if finite(self.vol_ystep) eq 0 then begin
             nogrid[1] = 1 
          endif else begin
             if self.vol_ystep eq 0 then nogrid[1] = 1 
          endelse
          if nogrid[1] then uniq_E_y = ydat[uniq(ydat,sort(ydat))] 
       endif
       if is_E_z then begin
          if finite(self.vol_zstep) eq 0 then begin
             nogrid[2] = 1 
          endif else begin
             if self.vol_zstep eq 0 then nogrid[2] = 1 
          endelse
          if nogrid[2] then uniq_E_z = zdat[uniq(zdat,sort(zdat))] 
       endif
    endif else begin     
       idat = (*self.projStrPtr).qty
       itit = self.intn_lab
       ndat = n_elements(idat)
       if arg_present(ierr) then begin
          ierr    = (*self.projStrPtr).err
          ok_ierr = 1b
       endif else ok_ierr = 0b

       ;ewid
       if self.eint_yn and (self.samp_typ ne 2) then $
          self->dm_choose_trajdata,-2,data=ewid
       ok_ewid = (n_elements(ewid) eq ndat)
       
       if self.extravaxis_yn then ids = [0,1,2,3] else ids = [0,1,3]
       viewlbs = self.traj_type[ids] 
    
       ;choose x, y, and z data
       self->dm_choose_trajdata,ids[self.volsel_x],data=xdat,label=xtit,debug=debug,is_E=is_E_x
       self->dm_choose_trajdata,ids[self.volsel_y],data=ydat,label=ytit,debug=debug,is_E=is_E_y
       self->dm_choose_trajdata,ids[self.volsel_z],data=zdat,label=ztit,debug=debug,is_E=is_E_z
       if is_E_x then begin
          if finite(self.vol_xstep) eq 0 then begin
             nogrid[0] = 1 
          endif else begin
             if self.vol_xstep eq 0 then nogrid[0] = 1 
          endelse
          if nogrid[0] then uniq_E_x = xdat[*,0,0]
       endif
       if is_E_y then begin
          if finite(self.vol_ystep) eq 0 then begin
             nogrid[1] = 1 
          endif else begin
             if self.vol_ystep eq 0 then nogrid[1] = 1 
          endelse
          if nogrid[1] then uniq_E_y = ydat[*,0,0]
       endif
       if is_E_z then begin
          if finite(self.vol_zstep) eq 0 then begin
             nogrid[2] = 1 
          endif else begin
             if self.vol_zstep eq 0 then nogrid[2] = 1 
          endelse
          if nogrid[2] then uniq_E_z = zdat[*,0,0]
       endif
       
       ;thickness
       nitem = [0,3,2,3]+self.extravaxis_yn
       if nitem[self.samp_typ] gt 3 then begin
          tran_invalid = 0b
          if (total(finite(self.volsel_tft1)) ne 0) then begin
             self->dm_choose_trajdata,ids[self.volsel_t1],index=index,range=self.volsel_tft1,empty=empty,debug=debug
             if empty then begin
                emptyitem = viewlbs[self.volsel_t1]
                index = -1 & tmp = temporary(index)
                tran_invalid = 1b
             endif else if n_elements(index) eq ndat then begin
                index = -1 & tmp = temporary(index)
             endif
          endif
          if n_elements(index) ne 0 then begin
             xdat = xdat[index]
             ydat = ydat[index]
             zdat = zdat[index]
             idat = idat[index]
             if ok_ierr then ierr = ierr[index]
             if ok_ewid then ewid = ewid[index]
          endif
          index = -1
       endif
    endelse

    if n_elements(emptyitem) ne 0 then $
       ok = dialog_message(['The specified '+emptyitem+' range covers no data.', $
               'The entire possible range is used instead.'],dialog_parent=self.tlb)

    ;x,y,z range
    n_xran = total(finite(self.vol_xran))
    n_yran = total(finite(self.vol_yran))
    n_zran = total(finite(self.vol_zran))
    if n_xran eq 2 then begin
       if self.vol_xran[1] gt self.vol_xran[0] then begin
          index = where((xdat le self.vol_xran[1]) and (xdat ge self.vol_xran[0]),count)
          if count gt 0 then begin
             xdat = xdat[index]
             ydat = ydat[index]
             zdat = zdat[index]
             idat = idat[index]
             if ok_ierr then ierr = ierr[index]
             index = -1
          endif
       endif
    endif
    if n_yran eq 2 then begin
       if self.vol_yran[1] gt self.vol_yran[0] then begin
          index  = where((ydat le self.vol_yran[1]) and (ydat ge self.vol_yran[0]),count)
          if count gt 0 then begin
             xdat = xdat[index]
             ydat = ydat[index]
             zdat = zdat[index]
             idat = idat[index]
             if ok_ierr then ierr = ierr[index]
             index = -1
          endif
       endif
    endif
    if n_zran eq 2 then begin
       if self.vol_zran[1] gt self.vol_zran[0] then begin
          index  = where((zdat le self.vol_zran[1]) and (zdat ge self.vol_zran[0]),count)
          if count gt 0 then begin
             xdat = xdat[index]
             ydat = ydat[index]
             zdat = zdat[index]
             idat = idat[index]
             if ok_ierr then ierr = ierr[index]
             index = -1
          endif
       endif
    endif
    if n_xran lt 2 then begin
       if finite(self.vol_xran[0]) then begin
          index = where(xdat ge self.vol_xran[0],count)
          if count gt 0 then begin
             xdat = xdat[index]
             ydat = ydat[index]
             zdat = zdat[index]
             idat = idat[index]
             if ok_ierr then ierr = ierr[index]
             index = -1
          endif
       endif
       if finite(self.vol_xran[1]) then begin
          index = where(xdat le self.vol_xran[1],count)
          if count gt 0 then begin
             xdat = xdat[index]
             ydat = ydat[index]
             zdat = zdat[index]
             idat = idat[index]
             if ok_ierr then ierr = ierr[index]
             index = -1
          endif
       endif
    endif
    if n_yran lt 2 then begin
       if finite(self.vol_yran[0]) then begin
          index = where(ydat ge self.vol_yran[0],count)
          if count gt 0 then begin
             xdat = xdat[index]
             ydat = ydat[index]
             zdat = zdat[index]
             idat = idat[index]
             if ok_ierr then ierr = ierr[index]
             index = -1
          endif
       endif
       if finite(self.vol_yran[1]) then begin
          index = where(ydat le self.vol_yran[1],count)
          if count gt 0 then begin
             xdat = xdat[index]
             ydat = ydat[index]
             zdat = zdat[index]
             idat = idat[index]
             if ok_ierr then ierr = ierr[index]
             index = -1
          endif
       endif
    endif
    if n_zran lt 2 then begin
       if finite(self.vol_zran[0]) then begin
          index = where(zdat ge self.vol_zran[0],count)
          if count gt 0 then begin
             xdat = xdat[index]
             ydat = ydat[index]
             zdat = zdat[index]
             idat = idat[index]
             if ok_ierr then ierr = ierr[index]
             index = -1
          endif
       endif
       if finite(self.vol_zran[1]) then begin
          index = where(zdat le self.vol_zran[1],count)
          if count gt 0 then begin
             xdat = xdat[index]
             ydat = ydat[index]
             zdat = zdat[index]
             idat = idat[index]
             if ok_ierr then ierr = ierr[index]
             index = -1
          endif
       endif
    endif

    ;rebinning according to the steps
    xmax  = max(xdat,min=xmin)
    ymax  = max(ydat,min=ymin)
    zmax  = max(zdat,min=zmin)
    xstep = (xmax-xmin)/200.  ;default in case of xstep absent
    ystep = (ymax-ymin)/200.  ;default in case of ystep absent
    zstep = (zmax-zmin)/200.  ;default in case of zstep absent
    nuex  = n_elements(uniq_E_x)
    nuey  = n_elements(uniq_E_y)
    nuez  = n_elements(uniq_E_z)
    if nogrid[0] and (nuex ge 2) then xstep = max([xstep,mean(uniq_E_x[1:nuex-1]-uniq_E_x[0:nuex-2])])
    if nogrid[1] and (nuey ge 2) then ystep = max([ystep,mean(uniq_E_y[1:nuey-1]-uniq_E_y[0:nuey-2])])
    if nogrid[2] and (nuez ge 2) then zstep = max([zstep,mean(uniq_E_z[1:nuez-1]-uniq_E_z[0:nuez-2])])
    if finite(self.vol_xstep) then begin
       if (self.vol_xstep ne 0) then xstep = abs(self.vol_xstep)
    endif
    if finite(self.vol_ystep) then begin
       if (self.vol_ystep ne 0) then ystep = abs(self.vol_ystep)
    endif
    if finite(self.vol_zstep) then begin
       if (self.vol_zstep ne 0) then zstep = abs(self.vol_zstep)
    endif

    checkfinite = ((self.ftype eq 1) or (self.ftype eq 2) or (strlen(self.userf_str) ne 0))
    dm_volumegrid_bin,xstep,ystep,zstep,xdat,ydat,zdat,idat,ierr=ierr,avgsum=self.bin_avgsum,$
       bintr=self.binthreshold,/conststep,checkfinite=checkfinite,extlib=self.bin_extlib,debug=debug

    ;smooth regular grid data
    index  = where(~finite(idat),count)     ;bins with no data
    refill = (count ne 0)                   ;flag for refilling
    if self.vol_smooth gt 0 then begin      ;smooth data only on plotting
       tmp = size(idat,/dimension)
       if (n_elements(tmp) ge 2) and (min(tmp) gt 3) then begin
          if ok_ierr then ierr = ierr^2
          if n_elements(tmp) eq 3 then begin
             vol_sm_kernel = fltarr(3,3,3)+self.disp_sm_kernel[0,0]
             vol_sm_kernel[*,*,1] = self.disp_sm_kernel
             vol_sm_kernel[1,1,[0,2]] = self.disp_sm_kernel[0,1]
          endif else $
             vol_sm_kernel = self.disp_sm_kernel
          if float(!version.release) ge 6.2 then begin
             for i=0, self.vol_smooth-1 do begin
                 idat = convol(idat,vol_sm_kernel,/edge_truncate,/nan,/normalize)
                 if ok_ierr then ierr = convol(ierr,vol_sm_kernel,/edge_truncate,/nan,/normalize)
                 if refill then begin
                    idat[index] = !values.f_nan
                    if ok_ierr then ierr[index] = !values.f_nan 
                 endif
             endfor
          endif else begin
             for i=0, self.vol_smooth-1 do begin
                 ;put 0 in bins with no data
                 if refill then begin
                    idat[index] = 0.0
                    if ok_ierr then ierr[index] = 0.0
                 endif
                 idat = convol(idat,vol_sm_kernel,total(vol_sm_kernel),/edge_truncate)
                 if ok_ierr then ierr = convol(ierr,vol_sm_kernel,total(vol_sm_kernel),/edge_truncate)
             endfor
             if refill then begin
                idat[index] = !values.f_nan
                if ok_ierr then ierr[index] = !values.f_nan
             endif
          endelse
          if ok_ierr then ierr = sqrt(ierr)
       endif
    endif
    index = -1
    ;intensity range
    oldimax  = max(idat,min=oldimin,/nan)
    imax = oldimax & imin = oldimin
    if finite(self.vol_iran[0]) then $
       if self.vol_iran[0] lt oldimax then imin=self.vol_iran[0]
    if finite(self.vol_iran[1]) then $
       if self.vol_iran[1] gt oldimin then imax=self.vol_iran[1]
    iran = [imin,imax]
    
    tmp = size(idat,/dimension)
    if n_elements(tmp) ne 3 then begin
       nx = n_elements(xdat) & ny = n_elements(ydat) & nz = n_elements(zdat)
       idat = reform(idat,[nx,ny,nz])
       if ok_ierr then ierr = reform(ierr,[nx,ny,nz])
    endif
    
    if keyword_set(debug) then print,'&&choose volume data finished in ',systime(/sec)-current,' sec.'
end