; $Id: dm_combine.pro,v 1.1 2010/05/11 21:47:23 ymqiu Exp $
;#######################################################################
;
; NAME:
;  dm_combine
;
; PURPOSE:
;  This program finds out the combined elements in array1 and array2
;
; CATEGORY:
;  general
;
; AUTHOR:
;  Yiming Qiu
;  NIST Center for Neutron Research
;  100 Bureau Drive, Gaithersburg, MD 20899-6102
;  United States
;  yiming.qiu@nist.gov
;  May, 2010
;
; LICENSE:
;  The software in this file is written by an employee of
;  National Institute of Standards and Technology
;  as part of the DAVE software project.
;
;  The DAVE software package is not subject to copyright protection
;  and is in the public domain. It should be considered as an
;  experimental neutron scattering data reduction, visualization, and
;  analysis system. As such, the authors assume no responsibility
;  whatsoever for its use, and make no guarantees, expressed or
;  implied, about its quality, reliability, or any other
;  characteristic. The use of certain trade names or commercial
;  products does not imply any endorsement of a particular product,
;  nor does it imply that the named product is necessarily the best
;  product for the stated purpose. We would appreciate acknowledgment
;  if the DAVE software is used or if the code in this file is
;  included in another product.
;
;#######################################################################

function dm_combine,arr1,arr2,count,index=index,no_copy=no_copy,debug=debug
    if keyword_set(debug) then current=systime(/sec)
    if keyword_set(index) then begin
       pos_arr1 = where(arr1 ge 0,count1)
       pos_arr2 = where(arr2 ge 0,count2)
       if (count1 eq 0) then begin
          count = count2  
          if keyword_set(no_copy) then begin    ;destroy arr1 and arr2
             n_arr = temporary(arr1)
             n_arr = temporary(arr2)
          endif else n_arr = arr2
          if keyword_set(debug) then print,'dm_combine finished in ',systime(/sec)-current,' sec.'
          return,n_arr
       endif
       if (count2 eq 0) then begin
          count = count1  
          if keyword_set(no_copy) then begin    ;destroy arr1 and arr2
             n_arr = temporary(arr2)
             n_arr = temporary(arr1)
          endif else n_arr = arr1
          if keyword_set(debug) then print,'dm_combine finished in ',systime(/sec)-current,' sec.'
          return,n_arr
       endif
       n_arr = [arr1[pos_arr1],arr2[pos_arr2]]
    endif else $
       n_arr = [arr1,arr2]
    n_arr = n_arr[uniq(n_arr,sort(n_arr))] & count = n_elements(n_arr)
    if keyword_set(no_copy) then begin ;destroy arr1 and arr2 after this function
       arr1 = -1 & tmp = temporary(arr1)
       arr2 = -1 & tmp = temporary(arr2)
    endif 
    if keyword_set(debug) then print,'dm_combine finished in ',systime(/sec)-current,' sec.'
    return,n_arr
end
