; $Id: dm_load_parameter.pro,v 1.25 2010/08/23 16:27:01 ymqiu Exp $
;#######################################################################
;
; NAME:
;  dm_load_parameter
;
; PURPOSE:
;  load parameter file
;
; CATEGORY:
;  dcs_mslice
;
; AUTHOR:
;  Yiming Qiu
;  NIST Center for Neutron Research
;  100 Bureau Drive, Gaithersburg, MD 20899-6102
;  United States
;  yiming.qiu@nist.gov
;  August, 2010
;
; LICENSE:
;  The software in this file is written by an employee of
;  National Institute of Standards and Technology
;  as part of the DAVE software project.
;
;  The DAVE software package is not subject to copyright protection
;  and is in the public domain. It should be considered as an
;  experimental neutron scattering data reduction, visualization, and
;  analysis system. As such, the authors assume no responsibility
;  whatsoever for its use, and make no guarantees, expressed or
;  implied, about its quality, reliability, or any other
;  characteristic. The use of certain trade names or commercial
;  products does not imply any endorsement of a particular product,
;  nor does it imply that the named product is necessarily the best
;  product for the stated purpose. We would appreciate acknowledgment
;  if the DAVE software is used or if the code in this file is
;  included in another product.
;
;#######################################################################

pro dcs_mslice::dm_load_parameter,parmfile,init=init,debug=debug
    WIDGET_CONTROL,/HOURGLASS
    openr,unit,parmfile,/get_lun
    tmp = ''
    ;catch and ignore all errors in this program
    catch, myerror
    if myerror ne 0 then begin
       catch,/cancel
       ok=dialog_message(!error_state.msg,/error)
       return
    end
    if self.samp_typ eq -1 then begin
       n_item = n_elements((*self.dataPtr).label)
       while(~ eof(unit)) do begin
          readf,unit,tmp
          head = strmid(tmp,0,8)
          tmp  = strmid(tmp,8,strlen(tmp)-8)
          case head of
            'samptyp:':begin
                       tmpvalue = dm_to_number(tmp,/int)
                       if tmpvalue ne -1 then begin
                          free_lun,unit
                          if (~ keyword_set(init)) then $
                             ok = dialog_message('Wrong type of parameter file.',/error,dialog_parent=self.tlb)
                          return
                       endif
                       end
            'datadir:':self.dirs[0]      = tmp
            'workdir:':self.dirs[1]      = tmp    
            'binmeth:':self.bin_avgsum   = dm_to_number(tmp,/int)
            'bincons:':self.bin_conststep= dm_to_number(tmp,/int)
            'bgcolor:':self.pbgcol_bw    = dm_to_number(tmp,/int)
            'volselx:':self.volsel_x     = dm_to_number(tmp,/int)
            'volsely:':self.volsel_y     = dm_to_number(tmp,/int)
            'volselz:':self.volsel_z     = dm_to_number(tmp,/int)
            'volx_fr:':self.vol_xran[0]  = dm_to_number(tmp,/float)
            'volx_to:':self.vol_xran[1]  = dm_to_number(tmp,/float)
            'volx_st:':self.vol_xstep    = dm_to_number(tmp,/float)
            'voly_fr:':self.vol_yran[0]  = dm_to_number(tmp,/float)
            'voly_to:':self.vol_yran[1]  = dm_to_number(tmp,/float)
            'voly_st:':self.vol_ystep    = dm_to_number(tmp,/float)
            'volz_fr:':self.vol_zran[0]  = dm_to_number(tmp,/float)
            'volz_to:':self.vol_zran[1]  = dm_to_number(tmp,/float)
            'volz_st:':self.vol_zstep    = dm_to_number(tmp,/float)
            'voli_fr:':self.vol_iran[0]  = dm_to_number(tmp,/float)
            'voli_to:':self.vol_iran[1]  = dm_to_number(tmp,/float)
            'vol_smo:':self.vol_smooth   = dm_to_number(tmp,/int)
            'volintp:':self.vol_interp   = dm_to_number(tmp,/int)
            'vol_col:':self.vol_color    = dm_to_number(tmp,/int)
            'vol_slt:':if n_item gt 5 then begin
                       tmpvalue          = dm_to_number(strsplit(tmp,'&',/extract,/preserve),/int)
                       n_tmp1            = n_elements(tmpvalue)
                       n_tmp2            = n_item-5
                       (*self.volsel_t)[0:((n_tmp1<n_tmp2)-1)] = tmpvalue[0:((n_tmp1<n_tmp2)-1)] 
                       endif
            'vol_tft:':if n_item gt 5 then begin
                       tmpvalue          = dm_to_number(strsplit(tmp,'&',/extract,/preserve))
                       n_tmp1            = n_elements(tmpvalue)/2
                       n_tmp2            = n_item-5
                       for i=0,(n_tmp1<n_tmp2)-1 do (*self.volsel_tft)[i,*] = tmpvalue[2*i:2*i+1]
                       endif 
            'disphfr:':self.disp_hran[0] = dm_to_number(tmp,/float)
            'disphto:':self.disp_hran[1] = dm_to_number(tmp,/float)
            'disphst:':self.disp_hstep   = dm_to_number(tmp,/float)
            'dispvfr:':self.disp_vran[0] = dm_to_number(tmp,/float)
            'dispvto:':self.disp_vran[1] = dm_to_number(tmp,/float)
            'dispvst:':self.disp_vstep   = dm_to_number(tmp,/float)
            'dispifr:':self.disp_iran[0] = dm_to_number(tmp,/float)
            'dispito:':self.disp_iran[1] = dm_to_number(tmp,/float)
            'dispsmo:':self.disp_smooth  = dm_to_number(tmp,/int)
            'dispker:':self.disp_sm_kernel[0:8] = dm_to_number(strsplit(tmp,'&',/extract,/preserve))
            'dispsha:':self.disp_shading = dm_to_number(tmp,/int)
            'dispcol:':self.disp_color   = dm_to_number(tmp,/int)
            'cut___x:':begin
                       self.cut_x[0:3]   = dm_to_number(strsplit(tmp,'&',/extract,/preserve),/int)
                       self.cut_x[0]     = ((n_item-3)<self.cut_x[0])
                       end
            'cut__fr:':self.cut_ran[0]   = dm_to_number(tmp,/float)
            'cut__to:':self.cut_ran[1]   = dm_to_number(tmp,/float)
            'cut__st:':self.cut_step     = dm_to_number(tmp,/float)       
            'cut___y:':begin
                       self.cut_y[0:3]   = dm_to_number(strsplit(tmp,'&',/extract,/preserve),/int)
                       self.cut_y[0]     = ((n_item-2)<self.cut_y[0])
                       end
            'cut_yfr:':self.cut_yran[0]  = dm_to_number(tmp,/float)
            'cut_yto:':self.cut_yran[1]  = dm_to_number(tmp,/float)
            'cut_sym:':self.cut_symbol   = dm_to_number(tmp,/int)
            'cut_col:':self.cut_color    = dm_to_number(tmp,/int)
            'userdef:':self.coldefine[0:2] = dm_to_number(strsplit(tmp,'&',/extract,/preserve),/int)
            'cutline:':self.cut_line     = dm_to_number(tmp,/int) 
            'dispslh:':self.dispsel_h    = ((n_item-3)<dm_to_number(tmp,/int)) 
            'dispslv:':self.dispsel_v    = ((n_item-3)<dm_to_number(tmp,/int)) 
            'dispslt:':if n_item gt 4 then begin
                       tmpvalue          = dm_to_number(strsplit(tmp,'&',/extract,/preserve),/int)
                       n_tmp1            = n_elements(tmpvalue)
                       n_tmp2            = n_item-4
                       (*self.dispsel_t)[0:((n_tmp1<n_tmp2)-1)] = tmpvalue[0:((n_tmp1<n_tmp2)-1)] 
                       endif
            'disptft:':if n_item gt 4 then begin
                       tmpvalue          = dm_to_number(strsplit(tmp,'&',/extract,/preserve))
                       n_tmp1            = n_elements(tmpvalue)/2
                       n_tmp2            = n_item-4
                       for i=0,(n_tmp1<n_tmp2)-1 do (*self.dispsel_tft)[i,*] = tmpvalue[2*i:2*i+1]
                       endif
            'cut_slt:':if n_item gt 3 then begin
                       tmpvalue          = dm_to_number(strsplit(tmp,'&',/extract,/preserve),/int)
                       n_tmp1            = n_elements(tmpvalue)
                       n_tmp2            = n_item-3
                       (*self.cutsel_t)[0:((n_tmp1<n_tmp2)-1)] = tmpvalue[0:((n_tmp1<n_tmp2)-1)] 
                       endif
            'cut_tft:':if n_item gt 3 then begin
                       tmpvalue          = dm_to_number(strsplit(tmp,'&',/extract,/preserve))
                       n_tmp1            = n_elements(tmpvalue)/2
                       n_tmp2            = n_item-3
                       for i=0,(n_tmp1<n_tmp2)-1 do (*self.cutsel_tft)[i,*] = tmpvalue[2*i:2*i+1] 
                       endif
            else:
          endcase
       endwhile 
       if (~ keyword_set(debug)) then begin
          self.colors[0] = (['white','black'])[self.pbgcol_bw]
          if (~ keyword_set(init)) then begin
             self->reset_display,/reset
             binSum   = widget_info(self.tlb,find_by_uname='binSum')
             binAvg   = widget_info(self.tlb,find_by_uname='binAvg')
             bincsYes = widget_info(self.tlb,find_by_uname='bincsYes')
             bincsNo  = widget_info(self.tlb,find_by_uname='bincsNo')
             bgBlack  = widget_info(self.tlb,find_by_uname='pbgBlack')
             bgWhite  = widget_info(self.tlb,find_by_uname='pbgWhite')
             dm_toggle_menubut,check=([binAvg,binSum])[self.bin_avgsum],uncheck=([binAvg,binSum])[1-self.bin_avgsum]
             dm_toggle_menubut,check=([bincsNo,bincsYes])[self.bin_conststep],uncheck=([bincsNo,bincsYes])[1-self.bin_conststep]
             dm_toggle_menubut,check=([bgBlack,bgWhite])[self.pbgcol_bw],uncheck=([bgBlack,bgWhite])[1-self.pbgcol_bw]
          endif
       endif               
    endif else begin
       ptr_free,self.absp_info         ;clear any existing self-shielding information
       clear_history = 1b
       newftype      = 0b
       while(~ eof(unit)) do begin
          readf,unit,tmp
          head = strmid(tmp,0,8)
          tmp  = strmid(tmp,8,strlen(tmp)-8)
          case head of
            'samptyp:':begin
                       tmpvalue = dm_to_number(tmp,/int)
                       if tmpvalue eq -1 then begin
                          free_lun,unit
                          if (~ keyword_set(init)) then $
                             ok = dialog_message('Wrong type of parameter file.',/error,dialog_parent=self.tlb)
                          return
                       endif
                       self.samp_typ     = tmpvalue
                       end
            'sshield:':self.absp_info    = ptr_new(dm_to_number(strsplit(tmp,'&',/extract,/preserve)))
            'in_geom:':self.instrgeom    = dm_to_number(tmp,/int)
            'f__type:':begin
                       old               = self.ftype
                       self.ftype        = dm_to_number(tmp,/int)
                       newftype          = (old ne self.ftype)
                       end         
            'dcsmask:':begin
                       tmpvalue = dm_to_number(strsplit(tmp,'&',/extract,/preserve),/int)
                       self.dcsdetmask[0:n_elements(tmpvalue)-1] = tmpvalue 
                       end
            'dcor_eq:':self.dcor_eqran[0:3] = dm_to_number(strsplit(tmp,'&',/extract,/preserve))
            'dcor_yn:':self.dcor_yn      = dm_to_number(tmp,/int)
            'evax_yn:':self.extravaxis_yn = dm_to_number(tmp,/int)
            'eemp_yn:':self.extraempcan_yn = dm_to_number(tmp,/int)
            'emp_fac:':self.empcanfactor = dm_to_number(strsplit(tmp,'&',/extract,/preserve))            
            'fold_rt:':self.foldright    = dm_to_number(tmp,/int)
            'edep_yn:':self.edep_yn      = dm_to_number(tmp,/int)
            'bitflip:':self.bitflip      = dm_to_number(tmp,/int)
            'dcs2thc:':self.dcs2thcorr   = dm_to_number(tmp,/int)
            'eadj_yn:':self.eadjust      = dm_to_number(tmp,/int)
            'eadj_sp:':self.eadjust_spec = dm_to_number(tmp,/float)
            'eadj_cf:':self.eadjust_checkfirst = dm_to_number(tmp,/int)
            'eint_yn:':self.eint_yn      = dm_to_number(tmp,/int)
            'selfshd:':self.ssfac        = dm_to_number(tmp,/float)
            'datadir:':begin 
                       self.dirs[0]      = tmp
                       if obj_valid(self.fileSel) then self.fileSel->set_path,tmp
                       end
            'workdir:':self.dirs[1]      = tmp
            'intnlab:':self.intn_lab     = tmp
            'intntyp:':self.intn_typ     = dm_to_number(tmp,/int)
            'binmeth:':self.bin_avgsum   = dm_to_number(tmp,/int)
            'bincons:':self.bin_conststep= dm_to_number(tmp,/int)
            'bgcolor:':self.pbgcol_bw    = dm_to_number(tmp,/int)
            'ebinsiz:':begin
                       tmpvalue          = dm_to_number(strsplit(tmp,'&',/extract,/preserve),/int)
                       if n_elements(tmpvalue) eq 3 then tmpvalue=[tmpvalue[1],tmpvalue[0],tmpvalue[2],1,1] $
                       else if n_elements(tmpvalue) eq 4 then tmpvalue=[tmpvalue,1]
                       self.e_bin        = tmpvalue
                       end
            'e_range:':self.e_range      = dm_to_number(strsplit(tmp,'&',/extract,/preserve))
            'mon_fit:':self.mon_sumfit   = dm_to_number(tmp,/int)
            'dcs_mon:':self.dcs_mon      = 0>(dm_to_number(tmp,/int))<2
            'ortharb:':self.view_ortharbi= dm_to_number(tmp,/int)
            'latt__a:':self.latt_parm[0] = dm_to_number(tmp,/double)
            'latt__b:':self.latt_parm[1] = dm_to_number(tmp,/double)
            'latt__c:':self.latt_parm[2] = dm_to_number(tmp,/double)
            'latt_aa:':self.latt_parm[3] = dm_to_number(tmp,/double)
            'latt_bb:':self.latt_parm[4] = dm_to_number(tmp,/double)
            'latt_cc:':self.latt_parm[5] = dm_to_number(tmp,/double)
            'orie_ux:':self.orie_u[0]    = dm_to_number(tmp,/float)
            'orie_uy:':self.orie_u[1]    = dm_to_number(tmp,/float)
            'orie_uz:':self.orie_u[2]    = dm_to_number(tmp,/float)
            'orie_vx:':self.orie_v[0]    = dm_to_number(tmp,/float)
            'orie_vy:':self.orie_v[1]    = dm_to_number(tmp,/float)
            'orie_vz:':self.orie_v[2]    = dm_to_number(tmp,/float)
            'oriepsi:':self.orie_psi     = dm_to_number(tmp,/float)
            'orieoff:':self.orie_offset  = dm_to_number(tmp,/float)
            'view1tp:':self.view_u1typ   = dm_to_number(tmp,/float)
            'view2tp:':self.view_u2typ   = dm_to_number(tmp,/float)
            'viewu1h:':self.view_u1[0]   = dm_to_number(tmp,/float)
            'viewu1k:':self.view_u1[1]   = dm_to_number(tmp,/float)
            'viewu1l:':self.view_u1[2]   = dm_to_number(tmp,/float)
            'view1lb:':begin
                       tmpvalue          = strsplit(tmp,'&',/extract,/preserve_null)
                       if n_elements(tmpvalue) eq 3 then begin ;old format parameter file
                          tmpvalue       = [tmpvalue[1],tmpvalue[0]]
                          if self.samp_typ lt 2 then self.samp_typ = 1-self.samp_typ
                       endif
                       self.view_u1lab   = tmpvalue
                       self.view_label[self.view_u1typ] = self.view_u1lab[0]
                       end
            'viewu1f:':self.view_u1fold  = dm_to_number(tmp,/int)
            'viewu1c:':self.view_u1foldcenter = dm_to_number(tmp,/float)           
            'viewu2h:':self.view_u2[0]   = dm_to_number(tmp,/float)
            'viewu2k:':self.view_u2[1]   = dm_to_number(tmp,/float)
            'viewu2l:':self.view_u2[2]   = dm_to_number(tmp,/float)
            'view2lb:':begin
                       tmpvalue          = strsplit(tmp,'&',/extract,/preserve_null)
                       if n_elements(tmpvalue) eq 3 then tmpvalue = [tmpvalue[1],tmpvalue[0]]
                       self.view_u2lab   = tmpvalue
                       self.view_label[self.view_u2typ] = self.view_u2lab[0]
                       end
            'viewu2f:':self.view_u2fold  = dm_to_number(tmp,/int)
            'viewu2c:':self.view_u2foldcenter = dm_to_number(tmp,/float)           
            'viewu3h:':self.view_u3[0]   = dm_to_number(tmp,/float)
            'viewu3k:':self.view_u3[1]   = dm_to_number(tmp,/float)
            'viewu3l:':self.view_u3[2]   = dm_to_number(tmp,/float)
            'view3lb:':self.view_u3lab   = tmp
            'viewu3f:':self.view_u3fold  = dm_to_number(tmp,/int)
            'viewu3c:':self.view_u3foldcenter = dm_to_number(tmp,/float)
            'volselx:':self.volsel_x     = dm_to_number(tmp,/int)
            'volsely:':self.volsel_y     = dm_to_number(tmp,/int)
            'volselz:':self.volsel_z     = dm_to_number(tmp,/int)
            'volx_fr:':self.vol_xran[0]  = dm_to_number(tmp,/float)
            'volx_to:':self.vol_xran[1]  = dm_to_number(tmp,/float)
            'volx_st:':self.vol_xstep    = dm_to_number(tmp,/float)
            'voly_fr:':self.vol_yran[0]  = dm_to_number(tmp,/float)
            'voly_to:':self.vol_yran[1]  = dm_to_number(tmp,/float)
            'voly_st:':self.vol_ystep    = dm_to_number(tmp,/float)
            'volz_fr:':self.vol_zran[0]  = dm_to_number(tmp,/float)
            'volz_to:':self.vol_zran[1]  = dm_to_number(tmp,/float)
            'volz_st:':self.vol_zstep    = dm_to_number(tmp,/float)
            'voli_fr:':self.vol_iran[0]  = dm_to_number(tmp,/float)
            'voli_to:':self.vol_iran[1]  = dm_to_number(tmp,/float)
            'vol_smo:':self.vol_smooth   = dm_to_number(tmp,/int)
            'volintp:':self.vol_interp   = dm_to_number(tmp,/int)
            'vol_col:':self.vol_color    = dm_to_number(tmp,/int)
            'vol_slt:':self.volsel_t1    = dm_to_number(tmp,/int)
            'vol_tft:':self.volsel_tft1  = dm_to_number(strsplit(tmp,'&',/extract,/preserve),/float)
            'dispper:':self.disp_perp    = dm_to_number(tmp,/int)
            'disppfr:':self.disp_pran[0] = dm_to_number(tmp,/float)
            'disppto:':self.disp_pran[1] = dm_to_number(tmp,/float)
            'disphfr:':self.disp_hran[0] = dm_to_number(tmp,/float)
            'disphto:':self.disp_hran[1] = dm_to_number(tmp,/float)
            'disphst:':self.disp_hstep   = dm_to_number(tmp,/float)
            'dispvfr:':self.disp_vran[0] = dm_to_number(tmp,/float)
            'dispvto:':self.disp_vran[1] = dm_to_number(tmp,/float)
            'dispvst:':self.disp_vstep   = dm_to_number(tmp,/float)
            'disptfr:':self.disp_tran[0] = dm_to_number(tmp,/float)
            'disptto:':self.disp_tran[1] = dm_to_number(tmp,/float)
            'dispifr:':self.disp_iran[0] = dm_to_number(tmp,/float)
            'dispito:':self.disp_iran[1] = dm_to_number(tmp,/float)
            'dispsmo:':self.disp_smooth  = dm_to_number(tmp,/int)
            'dispker:':self.disp_sm_kernel[0:8] = dm_to_number(strsplit(tmp,'&',/extract,/preserve))
            'dispsha:':self.disp_shading = dm_to_number(tmp,/int)
            'dispcol:':self.disp_color   = dm_to_number(tmp,/int)
            'cut___x:':begin
                       tmpvalue          = dm_to_number(strsplit(tmp,'&',/extract,/preserve),/int)
                       if n_elements(tmpvalue) eq 3 then tmpvalue=[tmpvalue[1],tmpvalue[0],tmpvalue[2],0]
                       self.cut_x        = tmpvalue
                       end
            'cut__fr:':self.cut_ran[0]   = dm_to_number(tmp,/float)
            'cut__to:':self.cut_ran[1]   = dm_to_number(tmp,/float)
            'cut__st:':self.cut_step     = dm_to_number(tmp,/float)
            'cutt1fr:':self.cut_t1ran[0] = dm_to_number(tmp,/float)
            'cutt1to:':self.cut_t1ran[1] = dm_to_number(tmp,/float)
            'cutt2fr:':self.cut_t2ran[0] = dm_to_number(tmp,/float)
            'cutt2to:':self.cut_t2ran[1] = dm_to_number(tmp,/float)
            'cutt3fr:':self.cut_t3ran[0] = dm_to_number(tmp,/float)
            'cutt3to:':self.cut_t3ran[1] = dm_to_number(tmp,/float)
            'cut___y:':begin
                       tmpvalue          = dm_to_number(strsplit(tmp,'&',/extract,/preserve),/int)
                       tmpid             = where(tmpvalue gt 7,count)
                       if count ne 0 then tmpvalue[tmpid] = 0
                       if n_elements(tmpvalue) eq 3 then tmpvalue=[tmpvalue[1],tmpvalue[0],tmpvalue[2],0]
                       self.cut_y        = tmpvalue
                       end
            'cut_yfr:':self.cut_yran[0]  = dm_to_number(tmp,/float)
            'cut_yto:':self.cut_yran[1]  = dm_to_number(tmp,/float)
            'cut_sym:':self.cut_symbol   = dm_to_number(tmp,/int)
            'cut_col:':self.cut_color    = dm_to_number(tmp,/int)
            'userdef:':self.coldefine[0:2] = dm_to_number(strsplit(tmp,'&',/extract,/preserve),/int)
            'cutline:':self.cut_line     = dm_to_number(tmp,/int)
            'traj__x:':begin
                       tmpvalue          = dm_to_number(strsplit(tmp,'&',/extract,/preserve),/int)
                       tmpid             = where(tmpvalue gt 4,count)
                       if count ne 0 then tmpvalue[tmpid] = 0
                       if n_elements(tmpvalue) eq 3 then tmpvalue=[tmpvalue[1],tmpvalue[0],tmpvalue[2],0]
                       self.traj_x       = tmpvalue
                       end
            'traj_xf:':self.traj_xran[0] = dm_to_number(tmp,/float)
            'traj_xt:':self.traj_xran[1] = dm_to_number(tmp,/float)
            'traj__y:':begin
                       tmpvalue          = dm_to_number(strsplit(tmp,'&',/extract,/preserve),/int)
                       tmpid             = where(tmpvalue gt 4,count)
                       if count ne 0 then tmpvalue[tmpid] = 1
                       if n_elements(tmpvalue) eq 3 then tmpvalue=[tmpvalue[1],tmpvalue[0],tmpvalue[2],1]
                       self.traj_y       = tmpvalue
                       end
            'traj_yf:':self.traj_yran[0] = dm_to_number(tmp,/float)
            'traj_yt:':self.traj_yran[1] = dm_to_number(tmp,/float)
            'traj_z1:':begin
                       tmpvalue          = dm_to_number(strsplit(tmp,'&',/extract,/preserve),/int)
                       self.traj_z1      = tmpvalue
                       end
            'trajz1f:':self.traj_z1ran[0] = dm_to_number(tmp,/float)
            'trajz1t:':self.traj_z1ran[1] = dm_to_number(tmp,/float)
            'traj_z2:':begin
                       tmpvalue          = dm_to_number(strsplit(tmp,'&',/extract,/preserve),/int)
                       self.traj_z2      = tmpvalue
                       end
            'trajz2f:':self.traj_z2ran[0] = dm_to_number(tmp,/float)
            'trajz2t:':self.traj_z2ran[1] = dm_to_number(tmp,/float)
            'eief___:':self.eief         = dm_to_number(tmp,/float)
            'estaend:':self.estartend    = dm_to_number(strsplit(tmp,'&',/extract,/preserve))
            'phxfile:':self.phxfile      = tmp
            'userfun:':self.userf_str    = tmp
            'history:':begin
                       if clear_history then begin
                          clear_history  = 0b
                          self.n_history = 0
                          totnum         = n_elements(self.file_hist)  ;make sure the capacity is not exceeded
                          ptr_free,self.file_hist
                       endif
                       self.file_hist[self.n_history mod totnum] = ptr_new(strsplit(tmp,string('A4'xb),/extract,/preserve))
                       if strmid((*(self.file_hist[self.n_history mod totnum]))[0],0,6) ne 'ftype_' then $
                          *(self.file_hist[self.n_history mod totnum]) = ['ftype_'+dm_to_string(self.ftype,/int),$
                               *(self.file_hist[self.n_history mod totnum])]
                       self.n_history = self.n_history+1
                       end
            else:
          endcase
       endwhile
       ptr_free,self.projStrPtr
       if (~ keyword_set(debug)) then begin
          self.colors[0] = (['white','black'])[self.pbgcol_bw]
          self->set_filehistory
          self->reset_file
          if newftype then begin
             tmp = self.ftype & self.ftype = -1
             ok  = self->dm_filetools('ftype_'+dm_to_string(tmp),init=init)
          endif else begin
             self->reset_menu
             if self.ftype eq 0 then begin
                ptr_free,self.detPosPtr
                self.detPosPtr = dm_load_detpos(/dcs,adjustdcs=self.dcs2thcorr)
             endif
          endelse
          if (~ keyword_set(init)) then $
             self->reset_pdt
          ;some warning messages   
          mesg = ''
          if (self.dcs_mon ne 1) and (self.ftype eq 0) then $
             mesg = [mesg,(['FC0','BM1','Duration'])[self.dcs_mon]+' is used as the monitor.']         
          if (self.dcs2thcorr eq 1) and (self.ftype eq 0) then $
             mesg = [mesg,'DCS 2theta Correction is On.']
          if self.eadjust ne 0 then $
             mesg = [mesg,'Elastic Peak Adjustment is On.']
          if n_elements(mesg) gt 1 then $
             ok = dialog_message(mesg[1:n_elements(mesg)-1],dialog_parent=self.tlb,title='Please note:')
       endif
    endelse
    free_lun,unit
end