; $Id: dm_plot_cut.pro,v 1.16.2.1 2010/09/16 15:41:23 ymqiu Exp $
;#######################################################################
;
; NAME:
;  dm_plot_cut
;
; PURPOSE:
;  plot the cut and save the data in a file if wanted
;
; CATEGORY:
;  dcs_mslice
;
; AUTHOR:
;  Yiming Qiu
;  NIST Center for Neutron Research
;  100 Bureau Drive, Gaithersburg, MD 20899-6102
;  United States
;  yiming.qiu@nist.gov
;  September, 2010
;
; LICENSE:
;  The software in this file is written by an employee of
;  National Institute of Standards and Technology
;  as part of the DAVE software project.
;
;  The DAVE software package is not subject to copyright protection
;  and is in the public domain. It should be considered as an
;  experimental neutron scattering data reduction, visualization, and
;  analysis system. As such, the authors assume no responsibility
;  whatsoever for its use, and make no guarantees, expressed or
;  implied, about its quality, reliability, or any other
;  characteristic. The use of certain trade names or commercial
;  products does not imply any endorsement of a particular product,
;  nor does it imply that the named product is necessarily the best
;  product for the stated purpose. We would appreciate acknowledgment
;  if the DAVE software is used or if the code in this file is
;  included in another product.
;
;#######################################################################

pro dcs_mslice::dm_plot_cut,overplot=overplot
    if (~ self->pd_valid()) then return
    WIDGET_CONTROL,/HOURGLASS
    
    ;select data
    self->dm_choose_cutdata,xdat=xdat,ydat=ydat,yerr=yerr,xtit=xtit,ytit=ytit,is_E_t=is_E_t,tran_invalid=tran_invalid
    if (~ self.errbar) and (n_elements(yerr) ne 0) then tmp=temporary(yerr)   ;destroy yerr

    ;y range
    yran = self.cut_yran
    if finite(yran[0]) eq 0 then begin
       yran[0] = min(ydat)
       yran[0] = yran[0]-0.1*abs(yran[0])
    endif
    if finite(yran[1]) eq 0 then begin
       yran[1] = max(ydat)
       yran[1] = yran[1]+0.1*abs(yran[1])
    endif
    symbol = self.symbols[self.cut_symbol]
    
    ;extra info in the title
    tmp_tit = ''
    tmp_leg = ''
    if self.samp_typ eq -1 then begin
       sel_list = (*self.dataPtr).label[2:(n_elements((*self.dataPtr).label)-1)]
       for i=0,n_elements(tran_invalid)-1 do begin
           if (tran_invalid[i] eq 0) and (total(finite((*self.cutsel_tft)[i,*])) ne 0) then $
              tmp_tit = tmp_tit+' '+sel_list[(*self.cutsel_t)[i]]+'=['+dm_to_string((*self.cutsel_tft)[i,0])+','+$
                 dm_to_string((*self.cutsel_tft)[i,1])+']';+(*self.dataPtr).unit[(*self.dispsel_t)[i]+2]
       endfor
    endif else begin
       viewlbs = [self.view_u1lab[(self.samp_typ ge 1)],self.view_u2lab[(self.samp_typ ge 1)],'E']
       if self.extravaxis_yn and (self.samp_typ eq 2) then viewlbs[2] = self.view_u3lab
       n_lbs  = [2,3,2+self.extravaxis_yn,3]
       cut_t2 = ((self.samp_typ eq 1) or ((self.samp_typ eq 2) and self.extravaxis_yn) or (self.samp_typ eq 3))    
       cut_t3 = ((self.samp_typ eq 1 or self.samp_typ eq 3) and self.extravaxis_yn)
       if (total(finite(self.cut_t1ran)) ne 0) and (~ keyword_set(tran_invalid[0])) then begin
          tmp_tit = ' '+viewlbs[(self.cut_x[self.samp_typ]+1) mod n_lbs[self.samp_typ]]+'=['+dm_to_string(self.cut_t1ran[0])+','+$
                    dm_to_string(self.cut_t1ran[1])+']'
          if is_E_t[0] then tmp_tit=tmp_tit+'meV'
       endif
       if cut_t2 and (total(finite(self.cut_t2ran)) ne 0) and (~ keyword_set(tran_invalid[1])) then begin
          tmp_tit = tmp_tit+' '+viewlbs[(self.cut_x[self.samp_typ]+2) mod n_lbs[self.samp_typ]]+'=['+dm_to_string(self.cut_t2ran[0])+','+$
                    dm_to_string(self.cut_t2ran[1])+']'
          if is_E_t[1] then tmp_tit=tmp_tit+'meV'
       endif
       if cut_t3 and (total(finite(self.cut_t3ran)) ne 0) and (~ keyword_set(tran_invalid[2])) then begin
          tmp_tit = tmp_tit+' '+self.view_u3lab+'=['+dm_to_string(self.cut_t3ran[0])+','+$
                    dm_to_string(self.cut_t3ran[1])+']'
          if is_E_t[2] then tmp_tit=tmp_tit+'meV'
       endif
       if (self.samp_typ eq 2) or ((self.samp_typ eq 3) and $
          ((keyword_set(is_E_t[0]) and keyword_set(tran_invalid[0])) or $
           (keyword_set(is_E_t[1]) and keyword_set(tran_invalid[1])) or $
           (keyword_set(is_E_t[2]) and keyword_set(tran_invalid[2])))) then begin
          case self.e_bin[self.samp_typ] of
             0:  tmp_tit = tmp_tit+''
             1:  tmp_tit = tmp_tit+' Elastic'
             2:  tmp_tit = tmp_tit+' E=['+dm_to_string(self.e_range[0])+','+dm_to_string(self.e_range[1])+']meV'
          endcase
       endif
       if (self.ftype eq 0) and (total(self.dcsdetmask[1:3]) ne 0) then begin  ;add mask info for DCS
          ind = where(1-self.dcsdetmask[1:3],count)
          if count gt 0 then begin
             tmp_leg = tmp_leg+'  '+(['Lower','Central','Upper'])[ind[0]]
             for i=1,count-1 do tmp_leg = tmp_leg+'+'+(['Lower','Central','Upper'])[ind[i]]
             tmp_leg = tmp_leg+' Bank'+(['','s'])[count gt 1]
          endif
       endif
    endelse
    title = self.titl_lab+tmp_tit
    legd  = self.legd_lab+tmp_leg

    ;plot the data
    isnew = 0b
    if self.useitool then begin
       ok = dialog_message('Itools plotting is not implemented yet. Soooorry!',dialog_parent=self.tlb)
       return
    endif else begin
       obj_name=  'dm_plot'
       if (~ obj_valid(self.current)) then begin
          self.current=obj_new(obj_name,xdat,ydat,yerr=yerr,xtit=xtit,ytit=ytit,parentobj=self,/isolatin1,$
                linestyle=self.lines[self.cut_line],psym=symbol,color=self.colors[self.cut_color],$
                path=self.dirs[1],xran=self.cut_ran,yran=yran,title=title,/no_copy,legend=legd,$
                background=(['black','white'])[self.pbgcol_bw],coldefine=self.coldefine)
          isnew = 1b
       endif else begin
          self.current->getproperty,keep=keep
          if keep eq 1 then begin
             isnew = 1b
             self.current=obj_new(obj_name,parentobj=self,background=(['black','white'])[self.pbgcol_bw],/isolatin1)
          endif
          if keyword_set(overplot) then $
             self.current->add_plot,xdat,ydat,yerr=yerr,/no_copy,legend=legd,coldefine=self.coldefine,$
                color=self.colors[self.cut_color],psym=symbol,linestyle=self.lines[self.cut_line] $
          else $
             self.current->setproperty,xdat=xdat,ydat=ydat,yerr=yerr,xtit=xtit,/no_copy,legend=legd,$
                ytit=ytit,xran=self.cut_ran,yran=yran,color=self.colors[self.cut_color],coldefine=self.coldefine,$
                psym=symbol,linestyle=self.lines[self.cut_line],title=title,/nodraw,hidelegend=0,bgcolor=self.pbgcol_bw
       endelse
       ;add powder lines in powder mode
       if (self.samp_typ eq 0) and ptr_valid(self.powd_line) and (isnew or (~ keyword_set(overplot))) then begin
          tmp = size(*self.powd_line)
          if tmp[0] eq 2 then tmp=1 else tmp=tmp[tmp[0]] 
          if ([self.view_u1typ, self.view_u2typ])[self.cut_x[0]] eq 1 then begin  ;x-axis is Q
             ;make sure elastic peak is in there
             if ([self.view_u1typ, self.view_u2typ])[(self.cut_x[0]+1) mod 2] eq 0 then begin
                maxE = max((*self.projStrPtr).en,min=minE)
                eran = [minE,maxE]
                if finite(self.cut_t1ran[0]) then eran[0] = self.cut_t1ran[0]
                if finite(self.cut_t1ran[1]) then eran[1] = self.cut_t1ran[1]
                if (eran[0] le 0) and (eran[1] ge 0) then ix = 0
             endif
          endif
          if n_elements(ix) ne 0 then $
             for i=0,tmp-1 do $
                 self.current->add_plot,(*self.powd_line)[*,ix,i],(*self.powd_line)[*,1-ix,i],$
                        psym='no symbol',color='dark yellow',linestyle='dashed',layer=i,$
                        legend=(['Al','Cu','Stainless Steel','Specified'])[(self.powd_typ-1)>0]+' powder line'
       endif
       self.current->draw
    endelse
    
    title=((self.samp_typ eq -1)?xtit+' vs. '+ytit:title)
    
    if keyword_set(overplot) and (~ isnew) then $
       self->dm_add_plotwindow,type=1,title=title+'[overplot]' $
    else $
       self->dm_add_plotwindow,type=1,title=title
end