; $Id: dm_plot_slice.pro,v 1.16.2.1 2010/09/16 15:41:23 ymqiu Exp $
;#######################################################################
;
; NAME:
;  dm_plot_slice
;
; PURPOSE:
;  plot the slice
;
; CATEGORY:
;  dcs_mslice
;
; AUTHOR:
;  Yiming Qiu
;  NIST Center for Neutron Research
;  100 Bureau Drive, Gaithersburg, MD 20899-6102
;  United States
;  yiming.qiu@nist.gov
;  September, 2010
;
; LICENSE:
;  The software in this file is written by an employee of
;  National Institute of Standards and Technology
;  as part of the DAVE software project.
;
;  The DAVE software package is not subject to copyright protection
;  and is in the public domain. It should be considered as an
;  experimental neutron scattering data reduction, visualization, and
;  analysis system. As such, the authors assume no responsibility
;  whatsoever for its use, and make no guarantees, expressed or
;  implied, about its quality, reliability, or any other
;  characteristic. The use of certain trade names or commercial
;  products does not imply any endorsement of a particular product,
;  nor does it imply that the named product is necessarily the best
;  product for the stated purpose. We would appreciate acknowledgment
;  if the DAVE software is used or if the code in this file is
;  included in another product.
;
;#######################################################################

pro dcs_mslice::dm_plot_slice,surfplot=surfplot
    if (~ self->pd_valid()) then return
    WIDGET_CONTROL,/HOURGLASS
    
    ;select data
    self->dm_choose_slicedata,xdat=xdat,ydat=ydat,zdat=zdat,xtit=xtit,ytit=ytit,ztit=ztit,zran=zran,zmin=zmin,$
          pran_invalid=pran_invalid,tran_invalid=tran_invalid

    ;extra info in the title
    tmp_tit = ''
    if (self.samp_typ eq -1) then begin
       sel_list = (*self.dataPtr).label[2:(n_elements((*self.dataPtr).label)-1)]
       for i=0,n_elements(tran_invalid)-1 do begin
           if (tran_invalid[i] eq 0) and (total(finite((*self.dispsel_tft)[i,*])) ne 0) then $
              tmp_tit = tmp_tit+' '+sel_list[(*self.dispsel_t)[i]]+'=['+dm_to_string((*self.dispsel_tft)[i,0])+','+$
                 dm_to_string((*self.dispsel_tft)[i,1])+']';+(*self.dataPtr).unit[(*self.dispsel_t)[i]+2]
       endfor
    endif else begin
       if (self.samp_typ eq 1) or ((self.samp_typ eq 2) and self.extravaxis_yn) or (self.samp_typ eq 3) then begin
          labels = [self.view_u1lab[1],self.view_u2lab[1],'E']
          if (self.samp_typ eq 2) and self.extravaxis_yn then labels[2] = self.view_u3lab
          if (total(finite(self.disp_pran)) ne 0) and (~ keyword_set(pran_invalid)) then begin
             tmp_tit = ' '+labels[self.disp_perp]+'=['+dm_to_string(self.disp_pran[0])+','+dm_to_string(self.disp_pran[1])+']'
             if labels[self.disp_perp] eq 'E' then tmp_tit=tmp_tit+'meV'
          endif
       endif
       if ((self.samp_typ eq 1) or (self.samp_typ eq 3)) and self.extravaxis_yn and $
          (~ keyword_set(tran_invalid)) and (total(finite(self.disp_tran)) ne 0) then $
          tmp_tit = tmp_tit+' '+self.view_u3lab+'=['+dm_to_string(self.disp_tran[0])+','+$
                    dm_to_string(self.disp_tran[1])+']'
       if (self.samp_typ eq 2) or ((self.samp_typ eq 3) and (strlen(tmp_tit) eq 0) and (self.disp_perp eq 2)) then begin
          case self.e_bin[self.samp_typ] of
             0:  tmp_tit = tmp_tit+''
             1:  tmp_tit = tmp_tit+' Elastic'
             2:  tmp_tit = tmp_tit+' E=['+dm_to_string(self.e_range[0])+','+dm_to_string(self.e_range[1])+']meV'
          endcase
       endif
       if (self.ftype eq 0) and (total(self.dcsdetmask[1:3]) ne 0) then begin  ;add mask info for DCS
          ind = where(1-self.dcsdetmask[1:3],count)
          if count gt 0 then begin
             tmp_tit = tmp_tit+'  '+(['Lower','Central','Upper'])[ind[0]]
             for i=1,count-1 do tmp_tit = tmp_tit+'+'+(['Lower','Central','Upper'])[ind[i]]
             tmp_tit = tmp_tit+' Bank'+(['','s'])[count gt 1]
          endif
       endif
    endelse
       
    title = self.titl_lab+tmp_tit

    ;plot the data
    if self.useitool then begin
       ok = dialog_message('Itools plotting is not implemented yet. Soooorry!',dialog_parent=self.tlb)
       return
    endif else begin
       obj_name = 'dm_plot'
       if (~ obj_valid(self.current)) then begin
          self.current=obj_new(obj_name,xdat,ydat,zdat,xtit=xtit,ytit=ytit,ztit=ztit,/no_copy,$
                parentobj=self,xran=self.disp_hran,yran=self.disp_vran,shading=self.disp_shading,$
                coltable=self.disp_color,iran=zran,title=title,zmin=zmin,surfplot=surfplot,$
                /usepolygon,/hidelegend,path=self.dirs[1],background=(['black','white'])[self.pbgcol_bw],/isolatin1)
       endif else begin
          self.current->getproperty,keep=keep
          if keep eq 1 then begin
             self.current = obj_new(obj_name,parentobj=self,background=(['black','white'])[self.pbgcol_bw],/isolatin1)
          endif
          self.current->setproperty,xdat=xdat,ydat=ydat,zdat=zdat,xtit=xtit,ytit=ytit,ztit=ztit,/no_copy,$
                xran=self.disp_hran,yran=self.disp_vran,shading=self.disp_shading,$
                coltable=self.disp_color,iran=zran,title=title,zmin=zmin,surfplot=surfplot,$
                /usepolygon,/nodraw,/hidelegend,bgcolor=self.pbgcol_bw
       endelse
       ;add powder lines in diffuse scattering mode and powder mode
       if ptr_valid(self.powd_line) then begin
          tmp = size(*self.powd_line) 
          if tmp[0] eq 2 then tmp=1 else tmp=tmp[tmp[0]]
          disp_perp = (self.disp_perp eq 2)
          if disp_perp then begin ;check if elastic is covered
             if finite(self.disp_pran[0]) then begin
                if self.disp_pran[0] gt 0 then disp_perp = 0b
             endif
             if finite(self.disp_pran[1]) then begin
                if self.disp_pran[1] lt 0 then disp_perp = 0b
             endif
          endif
          if ((self.samp_typ eq 1) or (self.samp_typ eq 3)) and disp_perp then begin
             ix = 0 
          endif else if (self.samp_typ eq 2) and ((~ self.extravaxis_yn) or (self.disp_perp eq 2)) then begin 
             ix = 0 
          endif
          if self.samp_typ eq 0 then begin  ;powder
             if self.view_u2typ eq 1 then ix = 1
             if self.view_u1typ eq 1 then ix = 0
          endif
          if n_elements(ix) ne 0 then $
             for i=0,tmp-1 do $
                 self.current->add_plot,(*self.powd_line)[*,ix,i],(*self.powd_line)[*,1-ix,i],psym='no symbol',$
                        color='dark yellow',linestyle='solid',layer=i,$
                        legend=(['Al','Cu','Stainless Steel','Specified'])[(self.powd_typ-1)>0]+' powder line'
       endif
       self.current->draw
    endelse
    self->dm_add_plotwindow,type=0,title=((self.samp_typ eq -1)?xtit+' vs. '+ytit:title)
end