; $Id: dm_plot_volume.pro,v 1.11.2.1 2010/09/16 15:41:23 ymqiu Exp $
;#######################################################################
;
; NAME:
;  dm_plot_volume
;
; PURPOSE:
;  plot the volume
;
; CATEGORY:
;  dcs_mslice
;
; AUTHOR:
;  Yiming Qiu
;  NIST Center for Neutron Research
;  100 Bureau Drive, Gaithersburg, MD 20899-6102
;  United States
;  yiming.qiu@nist.gov
;  September, 2010
;
; LICENSE:
;  The software in this file is written by an employee of
;  National Institute of Standards and Technology
;  as part of the DAVE software project.
;
;  The DAVE software package is not subject to copyright protection
;  and is in the public domain. It should be considered as an
;  experimental neutron scattering data reduction, visualization, and
;  analysis system. As such, the authors assume no responsibility
;  whatsoever for its use, and make no guarantees, expressed or
;  implied, about its quality, reliability, or any other
;  characteristic. The use of certain trade names or commercial
;  products does not imply any endorsement of a particular product,
;  nor does it imply that the named product is necessarily the best
;  product for the stated purpose. We would appreciate acknowledgment
;  if the DAVE software is used or if the code in this file is
;  included in another product.
;
;#######################################################################

pro dcs_mslice::dm_plot_volume
    if (~ self->pd_valid()) then return
    WIDGET_CONTROL,/HOURGLASS
    
    ;select data
    self->dm_choose_volumedata,xdat=xdat,ydat=ydat,zdat=zdat,idat=idat,xtit=xtit,ytit=ytit,ztit=ztit,iran=iran,$
          tran_invalid=tran_invalid
    ;extra info in the title
    tmp_tit = ''
    if (self.samp_typ eq -1) then begin
       sel_list = (*self.dataPtr).label[2:(n_elements((*self.dataPtr).label)-1)]
       for i=0,n_elements(tran_invalid)-1 do begin
           if (tran_invalid[i] eq 0) and (total(finite((*self.volsel_tft)[i,*])) ne 0) then $
              tmp_tit = tmp_tit+' '+sel_list[(*self.volsel_t)[i]]+'=['+dm_to_string((*self.volsel_tft)[i,0])+','+$
                 dm_to_string((*self.volsel_tft)[i,1])+']';+(*self.dataPtr).unit[(*self.volsel_t)[i]+2]
       endfor
    endif else begin
       if self.extravaxis_yn then ids = [0,1,2,3] else ids = [0,1,3]
       sel_list = ([self.view_u1lab[1],self.view_u2lab[1],self.view_u3lab,'E'])[ids]
        
       if n_elements(tran_invalid) ne 0 then begin
           if (tran_invalid eq 0) and (total(finite(self.volsel_tft1)) ne 0) then $
              tmp_tit = tmp_tit+' '+sel_list[self.volsel_t1]+'=['+dm_to_string(self.volsel_tft1[0])+','+$
                 dm_to_string(self.volsel_tft1[1])+']';+(*self.dataPtr).unit[(*self.volsel_t)[i]+2]
       endif
       
       if (self.ftype eq 0) and (total(self.dcsdetmask[1:3]) ne 0) then begin  ;add mask info for DCS
          ind = where(1-self.dcsdetmask[1:3],count)
          if count gt 0 then begin
             tmp_tit = tmp_tit+'  '+(['Lower','Central','Upper'])[ind[0]]
             for i=1,count-1 do tmp_tit = tmp_tit+'+'+(['Lower','Central','Upper'])[ind[i]]
             tmp_tit = tmp_tit+' Bank'+(['','s'])[count gt 1]
          endif
       endif
    endelse
       
    title = self.titl_lab+tmp_tit
    
    ;plot the data
    if self.useitool then begin
       ok = dialog_message('Itools plotting is not implemented yet. Soooorry!',dialog_parent=self.tlb)
       return
    endif else begin
       obj_name = 'dm_plot'
       if (~ obj_valid(self.current)) then begin
          self.current=obj_new(obj_name,xdat,ydat,zdat,idat,xtit=xtit,ytit=ytit,ztit=ztit,/no_copy,$
                parentobj=self,xran=self.vol_xran,yran=self.vol_yran,zran=self.vol_zran,iran=iran,$
                interpolate=self.vol_interp,coltable=self.vol_color,title=title,/hidelegend,path=self.dirs[1],$
                background=(['black','white'])[self.pbgcol_bw],/isolatin1)
       endif else begin
          self.current->getproperty,keep=keep
          if keep eq 1 then begin
             self.current = obj_new(obj_name,parentobj=self,background=(['black','white'])[self.pbgcol_bw],/isolatin1)
          endif
          self.current->setproperty,xdat=xdat,ydat=ydat,zdat=zdat,idat=idat,xtit=xtit,ytit=ytit,ztit=ztit,/no_copy,$
                xran=self.vol_xran,yran=self.vol_yran,zran=self.vol_zran,iran=iran,shading=self.disp_shading,$
                interpolate=self.vol_interp,coltable=self.vol_color,title=title,/nodraw,/hidelegend,bgcolor=self.pbgcol_bw
       endelse
       self.current->draw
    endelse
    self->dm_add_plotwindow,type=3,title=((self.samp_typ eq -1)?xtit+' vs. '+ytit+' vs. '+ztit:title)
end